Package: padr
Type: Package
Title: Quickly Get Datetime Data Ready for Analysis
Version: 0.6.2
Author: Edwin Thoen
Maintainer: Edwin Thoen <edwinthoen@gmail.com>
Description: Transforms datetime data into a format ready for analysis.
    It offers two core functionalities; aggregating data to a higher level interval
    (thicken) and imputing records where observations were absent (pad). 
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
Depends: R (>= 3.0.0)
Imports: Rcpp, dplyr (>= 1.0.0), lubridate, rlang
Suggests: ggplot2, testthat, knitr, rmarkdown, lazyeval, tidyr,
        data.table
RoxygenNote: 7.1.0
LinkingTo: Rcpp
VignetteBuilder: knitr
URL: https://github.com/EdwinTh/padr
BugReports: https://github.com/EdwinTh/padr/issues
ByteCompile: true
NeedsCompilation: yes
Packaged: 2022-11-23 15:08:44 UTC; edwinthoen
Repository: CRAN
Date/Publication: 2022-11-23 16:00:02 UTC
Built: R 4.4.1; x86_64-apple-darwin24.0.0; 2024-10-12 08:34:23 UTC; unix
